/*
  Copyright (c) 2007 Kathryn Huxtable

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  $Id: AttributeFormatterTest.java 1224 2007-04-17 14:04:24Z kathryn5 $
 */
package org.kathrynhuxtable.middleware.shibshim.attributes;

import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import junit.framework.TestCase;

/**
 * Test the SampleValidator class.
 */
public class AttributeFormatterTest extends TestCase {
    private int dateLength = 24;
    private String startAssertionExpected = "<Assertion IssueInstant=\"";
    private String dateRegexp = "\\d\\d\\d\\d-\\d\\d-\\d\\dT\\d\\d:\\d\\d:\\d\\d.\\d\\d\\dZ";
    private String endAssertionExpected =
        "\" AuthenticationInstant=\"foo\" Issuer=\"https://shim.example.edu/shibishim\" Handle=\"0123456789ABCDEF\">"
        + "<AttributeStatement>"
        + "<Attribute AttributeName=\"shib-em-ismemberof\">"
        + "<AttributeValue>example:group1</AttributeValue>"
        + "<AttributeValue>example:group2</AttributeValue>"
        + "<AttributeValue>example:group3</AttributeValue>"
        + "</Attribute>"
        + "<Attribute AttributeName=\"shib-ep-affiliation\">"
        + "<AttributeValue>employee@example.edu</AttributeValue>"
        + "<AttributeValue>member@example.edu</AttributeValue>"
        + "<AttributeValue>staff@example.edu</AttributeValue>"
        + "</Attribute>"
        + "<Attribute AttributeName=\"shib-ep-edupersonprincipalname\">"
        + "<AttributeValue>myusername@example.edu</AttributeValue>"
        + "</Attribute>"
        + "<Attribute AttributeName=\"shib-example-junk\">"
        + "<AttributeValue>A greater than&gt;here</AttributeValue>"
        + "<AttributeValue>A less than&lt;here</AttributeValue>"
        + "<AttributeValue>A null&#0;here</AttributeValue>"
        + "<AttributeValue>A quote&quot;here</AttributeValue>"
        + "<AttributeValue>A return&#13;here</AttributeValue>"
        + "<AttributeValue>An ampersand&amp;here</AttributeValue>"
        + "<AttributeValue>An apostrophe&apos;here</AttributeValue>"
        + "</Attribute>"
        + "</AttributeStatement>"
        + "</Assertion>";

	public void testFormatterSuccess() {
        Map attributes = new TreeMap();
        Set values = new TreeSet();
        values.add("example:group1");
        values.add("example:group2");
        values.add("example:group3");
        attributes.put("shib-em-ismemberof", values);
        values = new TreeSet();
        values.add("staff@example.edu");
        values.add("employee@example.edu");
        values.add("member@example.edu");
        attributes.put("shib-ep-affiliation", values);
        values = new TreeSet();
        values.add("myusername@example.edu");
        attributes.put("shib-ep-edupersonprincipalname", values);
        values = new TreeSet();
        values.add("A null\0here");
        values.add("A return\rhere");
        values.add("An ampersand&here");
        values.add("A less than<here");
        values.add("A greater than>here");
        values.add("An apostrophe'here");
        values.add("A quote\"here");
        attributes.put("shib-example-junk", values);
        
        String assertion = AttributeFormatter.formatAssertion(attributes, "foo", "https://shim.example.edu/shibishim", "0123456789ABCDEF");
        String startAssertion = assertion.substring(0, startAssertionExpected.length());
        assertEquals(startAssertionExpected, startAssertion);
        String date = assertion.substring(startAssertionExpected.length(), startAssertionExpected.length() + dateLength);
        assertTrue("Date is not valid", date.matches(dateRegexp));
        String endAssertion = assertion.substring(startAssertionExpected.length() + dateLength);
        assertEquals(endAssertionExpected, endAssertion);
	}
}

