/*
 Copyright (c) 2007 Kathryn Huxtable

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 $Id: ACSServletTest.java 1254 2007-04-18 12:22:02Z kathryn5 $
 */
package org.kathrynhuxtable.middleware.shibshim.web.acs;

import java.io.IOException;
import java.util.Vector;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import junit.framework.TestCase;

import org.easymock.MockControl;
import org.kathrynhuxtable.middleware.shibshim.util.properties.PropertiesBundle;

/**
 * Test the PropertiesBundle class.
 */
public class ACSServletTest extends TestCase {

    private MockControl         controlConfig;
    private ServletConfig       config;
    private MockControl         controlContext;
    private ServletContext      context;
    private MockControl         controlRequest;
    private HttpServletRequest  request;
    private MockControl         controlResponse;
    private HttpServletResponse response;
    private MockControl         controlSession;
    private HttpSession         session;

    private ACSServlet          servlet;

    public void setUp() {
        controlConfig = MockControl.createControl(ServletConfig.class);
        config = (ServletConfig) controlConfig.getMock();
        controlContext = MockControl.createControl(ServletContext.class);
        context = (ServletContext) controlContext.getMock();
        controlRequest = MockControl.createControl(HttpServletRequest.class);
        request = (HttpServletRequest) controlRequest.getMock();
        controlResponse = MockControl.createControl(HttpServletResponse.class);
        response = (HttpServletResponse) controlResponse.getMock();
        controlSession = MockControl.createControl(HttpSession.class);
        session = (HttpSession) controlSession.getMock();

        servlet = new ACSServlet();
    }

    public void tearDown() {
        controlConfig.verify();
        controlContext.verify();
        controlRequest.verify();
        controlResponse.verify();
        controlSession.verify();

        PropertiesBundle.clearForTesting();
    }

    public void testInitFailure() {
        config.getServletContext();
        controlConfig.setReturnValue(context);
        context.getInitParameter("propertiesFile");
        controlContext.setReturnValue("/unreadable.properties");

        controlConfig.replay();
        controlContext.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            servlet.init(config);
        } catch (ServletException e) {
            assertTrue(true);
            return;
        }
        fail("Failed to catch ServletException reading bad init");
    }

    public void testInitSuccess() {
        config.getServletContext();
        controlConfig.setReturnValue(context);
        context.getInitParameter("propertiesFile");
        controlContext.setReturnValue("/test.properties");

        controlConfig.replay();
        controlContext.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            servlet.init(config);
        } catch (ServletException e) {
            fail("Got ServletException reading good init: " + e);
        }
        assertTrue(true);
    }

    public void testLoginRedirect() {
        Vector headerNames = new Vector();
        headerNames.add("Shib-EM-isMemberOf");
        headerNames.add("Shib-EP-eduPersonAffiliation");
        Vector members = new Vector();
        members.add("example:group1");
        members.add("example:group2");
        members.add("example:group3");
        Vector affiliations = new Vector();
        affiliations.add("staff@example.edu");
        affiliations.add("employee@example.edu");
        affiliations.add("member@example.edu");
        MyDispatcher dispatcher = new MyDispatcher();

        config.getServletContext();
        controlConfig.setReturnValue(context);
        context.getInitParameter("propertiesFile");
        controlContext.setReturnValue("/test.properties");

        request.getRemoteAddr();
        controlRequest.setReturnValue("10.0.0.1");
        request.getHeader("Referer");
        controlRequest.setReturnValue("https://idp.example.edu/shibboleth-idp/SSO");
        request.getSession();
        controlRequest.setReturnValue(session);
        session.getId();
        controlSession.setReturnValue("0123456789ABCDEF");

        request.getRemoteUser();
        controlRequest.setReturnValue("myusername@example.edu", 3);
        request.getHeaderNames();
        controlRequest.setReturnValue(headerNames.elements());
        request.getHeaders("Shib-EM-isMemberOf");
        controlRequest.setReturnValue(members.elements());
        request.getHeaders("Shib-EP-eduPersonAffiliation");
        controlRequest.setReturnValue(affiliations.elements());
        
        request.getPathInfo();
        controlRequest.setReturnValue("/https/myapp.example.edu/myapp/localACS");
        request.setAttribute(null, null);
        controlRequest.setMatcher(MockControl.ALWAYS_MATCHER);
        request.getParameter("target");
        controlRequest.setReturnValue("/mypath");
        request.setAttribute(null, null);
        controlRequest.setMatcher(MockControl.ALWAYS_MATCHER);
        request.setAttribute(null, null);
        controlRequest.setMatcher(MockControl.ALWAYS_MATCHER);
        request.setAttribute(null, null);
        controlRequest.setMatcher(MockControl.ALWAYS_MATCHER);
        
        request.getRequestDispatcher("/WEB-INF/jsp/post.jsp");
        controlRequest.setReturnValue(dispatcher);

        controlConfig.replay();
        controlContext.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            servlet.init(config);
        } catch (ServletException e) {
            fail("Got ServletException reading good init: " + e);
        }

        try {
            servlet.doGet(request, response);
        } catch (IOException e) {
            fail("Got IOException calling servlet.doGet");
        } catch (ServletException e) {
            fail("Got ServletException calling servlet.doGet");
        }
        
        assertEquals(request, dispatcher.getRequest());
        assertEquals(response, dispatcher.getResponse());
    }
    
    public class MyDispatcher implements RequestDispatcher {
        private HttpServletRequest request = null;
        private HttpServletResponse response = null;

        public HttpServletRequest getRequest() {
            return request;
        }

        public HttpServletResponse getResponse() {
            return response;
        }

        public void include(ServletRequest request, ServletResponse response) {
            // Don't need this for this test.
        }

        public void forward(ServletRequest request, ServletResponse response) {
            this.request = (HttpServletRequest) request;
            this.response = (HttpServletResponse) response;
        }
    }
}
